# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls'
# for details.
#
# author: Torsten Irländer <torsten.irlaender@intevation.de>

import logging

try:
    import xml.etree.ElementTree as etree
except ImportError:
    import elementtree.ElementTree as etree

import traceback
from libmpuls.db.db import DB, get_db, CONFIG

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


GET_TAGS_SQL = """SELECT tag_id FROM nm_tags_cases_view WHERE case_id = %(id)s"""
ADD_TAG_SQL  = """SELECT create_nm_tag_case_ds(%(tagid)s, %(caseid)s)"""
DEL_TAG_SQL  = """SELECT delete_nm_tag_case_ds(%(tagid)s, %(caseid)s)"""

GET_OLD_CASES = """SELECT id from master_tbl_eval_total_view"""

GET_CASE_APPOINTMENTS = "SELECT id, master_id as case, sdatum as startdate, coalesce(edatum, sdatum) as enddate, name as summary, beschreibung as text, art as type FROM ka_fall_termine_tbl_view WHERE master_id = %(id)s AND coalesce(edatum, sdatum)::date+1 >= now() AND art= 0 UNION SELECT id, master_id as case, sdatum as startdate, coalesce(edatum, sdatum) as enddate, name as summary, beschreibung as text, art as type FROM ka_fall_termine_tbl_view WHERE art = 1"
GET_GLOBAL_APPOINTMENTS= "SELECT id, master_id as case, sdatum as startdate, coalesce(edatum, sdatum) as enddate, name as summary, beschreibung as text, art as type FROM ka_global_termine_tbl_view WHERE coalesce(edatum, sdatum)::date+1 >= now()"

class TagSetterConfig:

    def __init__(self, filename, username, password, database, host=None, port=None, caseid=None):

        self.dbname = database
        self.username = username
        self.password = password 
        self.host     = host
        self.port     = port
        self.caseid   = caseid # None means all cases

        try:
            xml          = open(filename, 'r')
            self.xml_doc = etree.parse(xml)
            xml.close()
        except StandardError, err:
            print 'Could not load xml tagsetter config: %s' % err

    def set_db(self, username, password, database, host=None, port=None):
        self.dbname = database
        self.username = username
        self.password = password 
        self.host     = host
        self.port     = port

    def _get_master_tbl(self):
        if self.caseid:
            return "SELECT * from master_tbl_eval_total_view WHERE id in (%s)" % self.caseid
        else:
            return "SELECT * from master_tbl_eval_total_view"

    def _get_check(self, id):
        for eval in self.xml_doc.findall('check'):
            tagid = eval.attrib.get('tagid')
            if str(id) == tagid:
                return eval

    def get_checks(self):
        '''
        Returns a list of ids of available checks.

        @return: list of ids of available checks 
        @rtype: List of integers 
        '''
        ids = []
        for eval in self.xml_doc.findall('check'):
            tagid = eval.attrib.get('tagid')
            if tagid:
                ids.append(tagid)
        return ids

    def get_name(self, id):
        config = self._get_check(id)
        return config.findtext('name')

    def get_desc(self, id):
        config = self._get_check(id)
        return config.findtext('desc')

    def get_addsql(self, id):
        config = self._get_check(id)
        sqlq = []
        table = self._get_master_tbl()
        for sql in config.findall('addtag'):
            sqlq.append(sql.findtext('sql') % {'table': table}) 
        return sqlq

    def get_delsql(self, id):
        config = self._get_check(id)
        sqlq = []
        table = self._get_master_tbl()
        for sql in config.findall('deltag'):
            sqlq.append(sql.findtext('sql') % {'table': table}) 
        return sqlq

class TagSetter:

    def __init__(self, config):
        self.config = config
        self.checks = self.config.get_checks()

    def perform(self, caseid=None):
        '''Check cases and set tags. If caseid is provided the caseid in the TagSetterConfig will be overwritten.'''
        if caseid:
            # Overwrite configcase id
            self.config.caseid = caseid
        for tagid in self.checks:
            self.del_tag(tagid)
            self.add_tag(tagid)

    def add_tag(self, tagid):
        sqllist   = self.config.get_addsql(tagid)
        conn, cur, count = None, None, 0
        db = get_db(
            self.config.username, 
            self.config.password, 
            self.config.dbname, 
            self.config.host, 
            self.config.port)
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                for sql in sqllist:
                    cur.execute(sql)
                    log.debug(cur.query)
                    for row in cur.fetchall():
                        cur.execute(ADD_TAG_SQL % {'tagid': tagid, 'caseid': row[0]})
                        log.debug(cur.query)
                        count += 1
                log.info('Added tag "%s" for for %s cases' % (tagid, count))
                conn.commit()
            except StandardError, err:
                print traceback.print_exc() 
                print "perform: %s %s"  % (err, cur.query)
        finally:
            db.recycleConnection(conn, cur)
            return count

    def del_tag(self, tagid):
        sqllist   = self.config.get_delsql(tagid)
        conn, cur, count = None, None, 0
        db = get_db(
            self.config.username, 
            self.config.password, 
            self.config.dbname, 
            self.config.host, 
            self.config.port)
        try:
            try:
                conn = db.getConnection()
                cur = conn.cursor()
                for sql in sqllist:
                    cur.execute(sql)
                    log.debug(cur.query)
                    for row in cur.fetchall():
                        cur.execute(DEL_TAG_SQL % {'tagid': tagid, 'caseid': row[0]})
                        log.debug(cur.query)
                        count += 1
                log.info('Deleted tag "%s" for for %s cases' % (tagid, count))
                conn.commit()
            except StandardError, err:
                print traceback.print_exc()
                print "perform: %s %s"  % (err, cur.query)
        finally:
            db.recycleConnection(conn, cur)
            return count
