# -*- coding: utf-8 -*-
"""\n
Do evaluations. Evaluations are configured in configfile. 
Connection details are set in /lib/db/db.py of this lib.

Usage: evalmpuls [options] configfile 

Options:
    -h / --help             Print this message and exit.
    -v                      Print sql statements which are sent to the database


    Datebase credentials. Default values are set in db.py

    -u                      username
    -P                      password
    -d                      database
    -H                      host
    -p                      port

    Evaluations options

    --start-date            YYYY-MM-DD
    --end-date              YYYY-MM-DD
    --start-date-field      Name of the field in the database
    --end-date-field        Name of the field in the database
    --evallist              Commaseparated list of evaluation ids. Default is
                            to evaluate all evaluations.
    --minphase              Set minimum phase a case must have to be considererd for evaluation.
    --sqlwhere              Additional SQL-WHERE statement, which can be
                            definded to reduce the total ammount of cases which
                            are taken for evaluation.
    --export                Output of the evaluation (XML, CVS, HTML). XML ist default.
    --agency                Name of the agency for which the evalution is done.
    --mapping               CSV file with mappings of agency names.

"""
# 20060403 Torsten Irlaender <torsten.irlaender@intevation.de>
#
# This program is free software under the GNU GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys 
import os
import getopt
import codecs
import datetime
import traceback
from libmpuls.evaluation.config import EvaluationConfig
from libmpuls.evaluation.evaluation import EvaluationSet 
from libmpuls.evaluation.export import EvaluationExportXML, EvaluationExportCSV, EvaluationExportHTML, EvaluationExportAdele
from libmpuls.db.db import CONFIG, get_db


def usage(code, msg=''):
    print >> sys.stderr, __doc__
    if msg:
        print >> sys.stderr, msg
    sys.exit(code)

def main():
    try:
        opts, args = getopt.getopt(
            sys.argv[1:], 
            'hvu:P:d:H:p:',
            ['help',
             'start-date=',
             'end-date=',
             'start-date-field=',
             'end-date-field=',
             'minphase=',
             'sqlwhere=',
             'evallist=',
             'agency=',
             'mapping=',
             'export='])
    except getopt.error, msg:
        usage(1, msg)


    # Set default values.
    username = CONFIG.get('login')
    password = CONFIG.get('password')
    database = CONFIG.get('dbname')
    host     = CONFIG.get('url')
    port     = CONFIG.get('port')

    sdatefield = None 
    sdate      = None
    edate      = None #datetime.date.today().strftime('%Y-%m-%d')
    edatefield = None 
    pphase     = None
    elist      = None
    sqlwhere   = None
    mapping    = None
    agency     = None
    exporter   = EvaluationExportXML() 
    verbose    = False

    for opt, arg in opts:
        if opt in ('-h', '--help'):
            usage(0)
        elif opt in ('-u'):
            username = arg
        elif opt in ('-P'):
            password = arg
        elif opt in ('-d'):
            database = arg
        elif opt in ('-H'):
            host = arg
        elif opt in ('-p'):
            port = arg
        elif opt in ('-v', '--verbose'):
            verbose = True
        elif opt in ('--start-date'):
            sdate = arg
        elif opt in ('--end-date'):
            edate = arg
        elif opt in ('--start-date-field'):
            sdatefield = arg
        elif opt in ('--end-date-field'):
            edatefield = arg
        elif opt in ('--minphase'):
            pphase = arg
        elif opt in ('--sqlwhere'):
            sqlwhere = arg
        elif opt in ('--evallist'):
            elist = arg.split(',')
        elif opt in ('--mapping'):
            mapping = arg
        elif opt in ('--agency'):
            agency = arg
        elif opt in ('--export'):
            if arg == 'csv':
                exporter = EvaluationExportCSV()
            if arg == 'html':
                exporter = EvaluationExportHTML()
            if arg == 'adele':
                exporter = EvaluationExportAdele()

    if len(args) < 1:
        usage(1)

    conn, cur = None, None
    db = get_db(username, password, database, host, port)
    try:
        try:
            conn = db.getConnection()
            config = EvaluationConfig(
                args[0], 
                conn, 
                sdate, 
                edate, 
                sdatefield, 
                edatefield, 
                pphase, 
                sqlwhere, 
                elist,
                agency,
                mapping)
            evaluationset = EvaluationSet(config, verbose)
            evaluationset.evaluate()
            result = evaluationset.export(exporter)
            print result.encode('utf-8')
        except Exception, err:
            #log.error('Error on evaluation set: %s' % err)
            traceback.print_exc()
            print "Failed to get connection. Check credetials"
    finally:
        db.recycleConnection(conn, cur)


if __name__ == '__main__':
    main()
